#!/usr/bin/env python

import click

from ytscraper.helper.echo import echov, echoe


@click.command()
@click.argument('term')
@click.pass_context
def fetch(context, term):
    """ Fetches subtitles from past queries."""
    # config = context['config']
    verbose = context.obj['verbose']
    echov("Starting YTcrawl's fetching mode.", verbose)
    echoe("Not implemented yet!")



#!/usr/bin/env python

import functools
import sys

import click


class color:
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'


class markup:
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'


info = markup.BOLD + color.PURPLE + "[STATUS] " + markup.END
usage = markup.BOLD + color.YELLOW + "[USAGE] " + markup.END
fail = markup.BOLD + color.RED + "[ERROR] " + markup.END
warning = markup.BOLD + color.YELLOW + "[WARNING] " + markup.END


def echov(text, verbose=True, nl=True):
    """ Wraps around click.echo and expands it by a `verbose` option.

    Parameter
    ---------
    text: str
        Text that is printed to standard output.
    verbose: {True, False}, optional
        Determines whether something is printed at all.
    nl: {True, False}, optional
        If set, a new line is added to the output.
    """
    if verbose:
        click.echo(info + text, nl=nl)


def echoe(text):
    """ Prints text to error output using click.echo and exits.

    Parameter
    ---------
    text: str
        Text that is printed to error output.
    """
    click.echo(fail + text, err=True)
    sys.exit(1)



#!/usr/bin/env python

from collections import deque
from pprint import pprint
from urllib import parse

import click

from ytscraper.helper.config import update_config
from ytscraper.helper.echo import echoe, echov
from ytscraper.helper.node import build_node_list
from ytscraper.helper.yt_api import get_search_videos, get_youtube_handle


@click.command()
@click.argument('search-type',
                default='query',
                type=click.Choice(['term', 'url', 'id']))
@click.argument('query', 
        nargs=1, 
        required=True)
@click.option('--number',
              '-n',
              multiple=True,
              type=click.IntRange(1, 50),
              help='Number of videos fetched per level.')
@click.option('--depth',
              '-d',
              help='Number of recursion steps.')
@click.option('--api-key',
              '-k',
              type=str,
              help='API Key to use YouTube API v3.')
@click.pass_context
def search(context, search_type, query, **options):
    """Searches YouTube using a specified query."""
    config = context.obj

    # CONFIGURATION
    echov("Updating configuration with command line options.", config['verbose'])
    update_config(config, options)
    echov("Done! Working with the following configuration:", config['verbose'])
    if config['verbose']:
        pprint(config)

    # AUTHENTICATION
    echov("Starting YouTube authentication.", config['verbose'])
    if 'api_key' not in config:
        echoe("""You need to provide an API key using `--api-key` 
        or the configuration file in order to query YouTube's API.
        Please see README on how to obtain such a key.""")
    handle = get_youtube_handle(config['api_key'])
    echov("API access established.", config['verbose'])

    # ARGUMENT PARSING
    if search_type == 'term':
        echov("Starting search using query {query}.", config['verbose'])
        start_ids = get_search_videos(handle, query, config['number'][0])
    elif search_type == 'id':
        echov("Starting search using video id {query}.", config['verbose'])
        start_ids = [query]
    elif search_type == 'url':
        echov("Starting search using the following video url:", config['verbose'])
        echov(query, config['verbose'])
        qterm = parse.urlsplit(query).query
        video_id = parse.parse_qs(qterm)['v']
        start_ids = [video_id]

    # QUERY
    node_list = build_node_list(handle, start_ids, 0, config['number'][0])
    node_queue = deque(node_list)
    processed_nodes = []
    while True:
        if len(node_queue) == 0:
            break
        node = node_queue.popleft()
        processed_nodes.append(node)
        echov('Current Video: ' + node.videoId, config['verbose'])
        if node.depth < config['depth']:
            # Clamp level_branch index to specified branch array.
            number = config['number'][max(0, min(node.depth + 1, len(config['number'])-1))]
            new_nodes = build_node_list(handle, node.relatedVideos,
                                        node.depth + 1, number)
            node_queue.extend(new_nodes)

    echov("Query finished! Result:")
    for node in processed_nodes:
        pprint(node)

    # TODO Save to file



#!/usr/bin/env python

from pprint import pprint

import click

from ytscraper.commands import fetch, search
from ytscraper.helper.config import load_config, update_config
from ytscraper.helper.echo import echov


@click.group()
@click.option('--config-path',
              '-c',
              type=click.Path(),
              help='YAML configuration file.')
@click.option('--verbose',
              '-v',
              is_flag=True,
              help='Show more verbose output.')
@click.pass_context
def entry_point(context, config_path, verbose):
    echov("Reading configuration file.", verbose)
    update_config(context.obj, load_config(config_path))
    echov("Read the following configuration:", verbose)
    if verbose:
        pprint(context.obj)

    context.obj['verbose'] = verbose


entry_point.add_command(search.search)
entry_point.add_command(fetch.fetch)

if __name__ == "__main__":
    entry_point(obj={})
